package tetris.model;

import tetris.Tetris;
import tetris.gui.Block;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * The class Field implements the playing field of a Tetris game.
 *
 * @author Stephan Fischli
 * @version 2.1
 */
public class Field {

	/**
	 * The blocks of the field.
	 */
	private final List<Block> blocks = new ArrayList<>();

	/**
	 * Gets the blocks of the field.
	 *
	 * @return the blocks of the field
	 */
	public List<Block> getBlocks() {
		return blocks;
	}

	/**
	 * Adds blocks to the field.
	 *
	 * @param blocks the blocks to add
	 */
	public void addBlocks(Block[] blocks) {
		this.blocks.addAll(Arrays.asList(blocks));
	}

	/**
	 * Removes all blocks from the field.
	 */
	public void clear() {
		blocks.clear();
	}

	/**
	 * Detects if the specified blocks collide with the border or the blocks of the field.
	 *
	 * @param blocks the blocks to check for collision
	 * @return true if one of the blocks collides, false otherwise
	 */
	public boolean detectCollision(Block[] blocks) {
		for (Block block : blocks) {
			if (block.x < 0 || block.x >= Tetris.WIDTH || block.y < 0 || block.y >= Tetris.HEIGHT) {
				return true;
			}
			for (Block fieldBlock : this.blocks) {
				if (block.x == fieldBlock.x && block.y == fieldBlock.y) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Removes the full rows from the field.
	 *
	 * @return the number of removed rows
	 */
	public int removeFullRows() {
		int numRows = 0;
		for (int y = Tetris.HEIGHT - 1; y >= 0; y--) {
			if (isRowFull(y)) {
				removeRow(y);
				numRows++;
			}
		}
		return numRows;
	}

	/**
	 * Checks if a row is full of blocks.
	 *
	 * @param y the y-coordinate of the row
	 * @return true if the row is full, false otherwise
	 */
	private boolean isRowFull(int y) {
		int numBlocks = 0;
		for (Block block : blocks) {
			if (block.y == y) {
				numBlocks++;
			}
		}
		return numBlocks == Tetris.WIDTH;
	}

	/**
	 * Removes the blocks of a row and shifts the blocks of the upper rows down.
	 *
	 * @param y the y-coordinate of the row
	 */
	private void removeRow(int y) {
		Iterator<Block> iter = blocks.iterator();
		while (iter.hasNext()) {
			Block block = iter.next();
			if (block.y == y) {
				iter.remove();
			} else if (block.y > y) {
				block.y--;
			}
		}
	}
}
