package tetris.model;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Scanner;

/**
 * The class Score implements the scoring of the Tetris game.
 *
 * @author Stephan Fischli
 * @version 2.1
 */
public class Scoring {

	/**
	 * The score rewards for the removal of full rows.
	 */
	private static final int[] SCORE_REWARDS = {0, 40, 100, 300, 1200};

	/**
	 * The number of rows needed to enter the next level.
	 */
	private static final int ROWS_PER_LEVEL = 10;

	/**
	 * The name of the high score file.
	 */
	private static final String HIGH_SCORE_FILE = "highscore.txt";

	/**
	 * The total number of removed rows.
	 */
	private int removedRows = 0;

	/**
	 * The current score of the game.
	 */
	private int score = 0;

	/**
	 * The high score of the game.
	 */
	private int highScore;

	/**
	 * Constructs a scoring and reads the high score file.
	 */
	public Scoring() {
		loadHighScore();
	}

	/**
	 * Gets the current level of the game.
	 *
	 * @return the current level of the game
	 */
	public int getLevel() {
		return 1 + removedRows / ROWS_PER_LEVEL;
	}

	/**
	 * Gets the current score of the game.
	 *
	 * @return the current score of the game
	 */
	public int getScore() {
		return score;
	}

	/**
	 * Gets the high score of the game.
	 *
	 * @return the high score of the game
	 */
	public int getHighScore() {
		return highScore;
	}

	/**
	 * Updates the score according to the number of removed rows.
	 *
	 * @param numRows the number of removed rows
	 */
	public void updateScore(int numRows) {
		removedRows += numRows;
		score += SCORE_REWARDS[numRows];
	}

	/**
	 * Updates the high score according to the current score.
	 */
	public void updateHighScore() {
		if (score > highScore) {
			highScore = score;
			saveHighScore();
		}
	}

	/**
	 * Resets the level and the score of the game.
	 */
	public void reset() {
		removedRows = 0;
		score = 0;
	}

	/**
	 * Loads the high score from the high score file.
	 */
	private void loadHighScore() {
		try (Scanner scanner = new Scanner(new FileInputStream(HIGH_SCORE_FILE))) {
			highScore = Integer.parseInt(scanner.nextLine());
		} catch (FileNotFoundException ex) {
			highScore = 0;
		} catch (NumberFormatException ex) {
			System.err.println("High score could not be read");
		}
	}

	/**
	 * Saves the high score to the high score file.
	 */
	private void saveHighScore() {
		try (PrintWriter writer = new PrintWriter(new FileWriter(HIGH_SCORE_FILE))) {
			writer.println(highScore);
		} catch (IOException ex) {
			System.err.println("High score could not be written");
		}
	}
}
